# TSTP Upgrade Guide

This document provides guidance to developers that need to upgrade an
existing TSTP configurations to work with recent versions of TSTP.

## Build System

The build system has been upgraded to be more strict, but it also
provides a new capability to work around any problems this exposes.

### All Programs Must Build

The build system now requires all C test programs to compile.
Previously, errors did not stop the build, so tests could fail to
be included in the final package or image.

Primarily, this means the `Makefile` no longer ignores build errors.
Tests that trigger errors will stop the build, to allow debugging and
fixing/reporting the problems.

### Reports All Warnings

The `Makefile` adds `-Wall -Wextra` to `CFLAGS`, ensuring the compiler
reports all possible warnings.

In most cases, the code that causes these warnings should be fixed,
rather than ignoring the errors that they produce.

In rare cases, certain warnings may be disabled for an individual
test by adding the appropriate flag to its individual CFLAGS.

### All Warnings Are Errors

The `Makefile` adds `-Werror` to `CFLAGS`, forcing the compiler to
treat all warnings as errors.  This ensures that minor problems do
not become major problems.

### Disabling Broken Tests

If some tests will not build due to missing target/platform
requirements, they may be disabled by providing the tests' names in
`TSTP_DISABLED_TESTS` when running `make`.

For example, `test_adc` requires a header file provided only
by i.MX Linux targets, so other targets will need to disable it:
```
make TSTP_DISABLED_TESTS="test_adc"
```

The build recipe provided by the `meta-timesys-test` layer should
permit setting that variable directly in an overlay, such that its value
will be passed directly to `oe_make` automatically.

## Test Harness

The main test harness (`Test.sh`) has received several upgrades:

- Moved configuration file location to `conf/`
- New test entry points for configuration files

### Configuration Files Moved

The `automated.conf` and `interactive.conf` files have been moved
into a new `conf/` subdirectory (e.g. from `/usr/tstp` to `/usr/tstp/conf`).

### New Test Entrypoints

The test harness now supports `run_automated` and `run_interactive`
entrypoints, in addition to the original `run_test` entry point.

The new entry points slightly reorder the arguments of `run_test`.
Primarily, they allow tests to be passed arguments that contain spaces,
but they also provide the foundation for future consolidation of the
configuration files.

```
run_automated <name> <program> [<args>]
run_interactive <name> <prompt> <question> <program> [<args>]
```

## Tests

A number of tests have been updated in potentially incompatible ways.

### Bash

A number of tests have ben rewritten to avoid using Bash features, in
order to allow them to be run on other shells.  

The goal here is to make all scripts work with `ash` provided by
`busybox`.  If they work there, they should also continue to work when
run under `bash`.

### CPU

The `test_cpufreq.sh` script now selects the `userspace` governor
before attempting to set the CPU frequency.

### I2C

A new `test_i2c.sh` script now provides a single entry point for
various I2C tests

|Old|New|Function|
|---|---|---|
|`i2c-test.sh`|`i2c_test.sh detect`|Detects all buses and devices|
|`i2c_read_verify.sh`|`i2c_test.sh read`|Read and verify register value|
|`i2c_send_command.sh`|`i2c_test.sh write`|Send a command and optionally read its reply|
|*none*|`i2c_test.sh dump`|Dumps all registers for a device|

### SD/MMC

A new `test_card.sh` script unifies the `sd-test.sh` and `mmc-test.sh`
scripts.  By default, the new script automatically detects the card type.

### Watchdog

The `watchdog-test.sh` script now provides full coverage of the
watchdog device.  Previously, it only provided a basic smoketest,
without actually exercising the driver.  It also tests the sysfs
interface to the Watchdog.  These extensions may expose new failures
in the system under test.
