#!/bin/bash
# log2md.sh: converts a test log tarball to GitLab Wiki Markdown
#
#  On target, create a tarball of the new log directory from /usr/tstp:
#     tar cf <timestamp>.tar logs/<timestamp>
#
#  On the host, run this script with the tarball and wiki repo path:
#     log2md.sh extract <timestamp>.tar <path/to/customer.wiki>
#
#  The log directory will be extracted, and all of the log files will
#  be converted into Markdown.  The test suite logs will be converted
#  to tables that link to the individual tests.

set -e

DEBUG=${DEBUG:-false}

run() { ! $DEBUG || echo "$*" >&2; "$@"; }
error() { echo "$*" >&2; exit 1; }

convert_header() {
	local in=$1
	local n=$2
	run echo -n "# "
	run head -n $n $in
}

convert_file() {
	local in=$1
	run convert_header $in 2
	run echo '```'
	run tail -n+3 $in
	run echo '```'
}

convert_index() {
	local in=$1
	run convert_header $in 5
	run echo '|Test|Result|'
	run echo '|----|----|'
	run tail -n+6 $in | run sed \
		-e 's,^\(.\+\)\s\+: \(.*\)$,|[\1](./\1)|\2|,' \
		-e 's,\s\+\([])]\),\1,g'
}

generate_overview() {
	local dir=$1
	shift

	local index
	echo "# Test Logs ($dir)"
	echo
	for index in "$@"; do
		echo "## [$index](./$index)"
		tail -n+3 $index.md
	done
}

do_extract() {
	local tarfile=$1
	local wikidir=$2
	[ "$tarfile" -a "$wikidir" ] || error "usage: $0 <tarball> <wikidir>"
	[ -f "$tarfile" ] || error "$tarfile: tarball does not exist"
	if [ ! -d "$wikidir/logs" ]; then
		error "$wikidir: directory does not contain logs/"
	fi
	local tarpath
	tarpath=$(realpath "$tarfile")

	run cd $wikidir
	run tar xf $tarpath

	local dirname
	dirname=${tarfile%.tar}
	run cd logs/$dirname

	local -a indexes
	local file md
	for file in *.log; do
		md="${file%.log}.md"
		if head -n 1 "$file" | grep -q ^Starting; then
			run convert_index "$file" >"$md"
			indexes=( "${indexes[@]}" "${md%.md}" )
		else
			md=$(echo $md | sed -e 's,_, ,g')
			run convert_file "$file" >"$md"
		fi
		rm $file
	done

	generate_overview $dirname "${indexes[@]}"  >Overview.md
}

do_create() {
	local logdir=$1
	[ "$logdir" ] || error "usage: $0 <logdir>"
	[ -d "logs" ] || error "$0 must be run in /usr/tstp"
	[ -d "$logdir" ] || error "$logdir: log directory does not exist"

	local tarfile
	tarfile=$(basename "$logdir").tar
	run tar cf $tarfile $logdir
}

do_cmd() {
	local cmd=$1
	[ "$cmd" ] || error "missing command"
	shift

	case "$cmd" in
	create)   do_create "$@" ;;
	extract)  do_extract "$@" ;;
	*)        error "$cmd: unknown command" ;;
	esac
}

do_cmd "$@"
