#!/usr/bin/python3
# log2xunit.py: converts TSTP log files into XUnit XML
#
#  On the target, create a tarball of the new log directory from /usr/tstp:
#     tar cf <timestamp>.tar logs/<timestamp>
#
#  On the host, run this script with the tarball and output path:
#     log2md.sh extract <timestamp>.tar <outdir>
#
#  The logs will be parsed directly out of the tarball, and the XUnit
#  result files (corresponding to each test suite) will be written to the
#  output directory.

import os
import sys
import argparse
import tarfile
import re
import html
import xml.etree.ElementTree as ET
import datetime


class Suites(object):
    def __init__(self):
        self.index = {}
        self.tests = {}


class State(object):
    def __init__(self):
        self.indexes = {}
        self.tests = {}


class TestResult(object):
    def __init__(self, name, result=None):
        self.name = name
        self.result = result
        self.start = None
        self.finish = None
        self.lines = []
        self.errors = []


class TestSuiteResult(object):
    def __init__(self, name):
        self.name = name
        self.start = None
        self.finish = None
        self.tests = []


class ParserError(RuntimeError):
    pass


class Parser(object):
    def __init__(self, state):
        self.buf = None
        self.state = state

    def dump(self):
        print(self.buf.read().decode("utf-8"))

    def readline(self):
        return self.buf.readline().decode("utf-8")

    def skiplines(self, n=1):
        while n > 0:
            self.readline()
            n -= 1

    def parse_buffer(self, name, buf):
        self.buf = buf
        name = os.path.basename(name)[:-4]

        try:
            line = self.readline().strip()
            if line.startswith('Starting '):
                    self.state.indexes[name] = self.parse_index(line)
            else:
                test = self.parse_test(line)
                self.state.tests[test.name] = test
        except ParserError as e:
            print("%s: parser error:\n%s" % (name, str(e)))
            sys.exit(1)

    def parse_index(self, hdr):
        match = re.match(r"^[^']*'([^']*)'", hdr)
        name = match[0]
        print("%s: parsing index" % match[1])
        self.skiplines(2)

        result = TestSuiteResult(match[1])
        result.start = self.parse_timestamp()

        self.skiplines()

        while True:
            name, outcome = self.parse_outcome(self.readline())
            if not name:
                break
            result.tests.append(TestResult(name, outcome))

        result.finish = self.parse_timestamp()

        return result

    def parse_outcome(self, line):
        match = re.match(r'([^:]+): (.*)', line)
        if match:
            return (match.group(1).strip(), match.group(2))
        else:
            return (None, None)

    def parse_timestamp(self):
        line = self.readline().strip()
        match = re.match(r'(Started|Finished): (.*)', line)
        if not match:
            raise ParserError("no timestamp found in '%s'" % line)
        return match[2]

    def parse_test(self, name):
        print("%s: parsing test" % name)
        self.skiplines()

        result = TestResult(name)
        result.start = self.readline().strip()
        self.skiplines(2)

        lines = self.buf.read().decode("utf-8", "ignore").split("\n")
        outcome, result.result = self.parse_outcome(lines[-2])
        if not outcome:
            raise ParserError("unable to parse outcome: " + lines[-2])
        result.lines = lines[:-5]

        return result


class CalculateTestTimes(object):
    def __init__(self, state):
        self.state = state

    def run(self):
        for s in self.state.indexes.values():
            self.convert_suite(s)

    def convert_suite(self, s):
        # first, fill in missing test finish times
        last = None
        for t in s.tests:
            t = self.state.tests[t.name]
            if last:
                last.finish = t.start
            last = t
        if last:
            last.finish = s.finish

        # then, convert time strings into datetime objects
        for t in s.tests:
            t = self.state.tests[t.name]
            self.convert_test_times(t)

        self.convert_test_times(s)

    def convert_test_times(self, t):
        t.start = self.convert_test_time(t.start)
        t.finish = self.convert_test_time(t.finish)
        t.elapsed = t.finish - t.start

    def convert_test_time(self, text):
        return datetime.datetime.strptime(text, '%a %b  %d %H:%M:%S UTC %Y')


class XUnitEmitter(object):
    def __init__(self, state, outdir):
        self.state = state
        self.outdir = outdir

    def emit(self):
        ass = ET.Element('testsuite', {'name': 'timesys-test-project'})

        for name in sorted(self.state.indexes.keys()):
            suite = self.state.indexes[name]
            self.emit_suite(suite)

    def emit_suite(self, suite):
        el = ET.Element('testsuite', {'name': suite.name})
        for t in suite.tests:
            t = self.state.tests[t.name]
            el.append(self.emit_test(suite, t))

        fname = os.path.join(self.outdir, suite.name + '.xml')
        print("%s: saving" % fname)

        tree = ET.ElementTree(el)
        tree.write(fname, encoding='UTF-8')

    def emit_test(self, suite, test):
        attrs = {'classname': suite.name, 'name': test.name}
        el = ET.Element('testcase', attrs)
        el.set('time', str(int(test.elapsed.total_seconds())))
        if test.result == 'FAILURE':
            el.append(self.emit_failure(test))
        return el

    def emit_failure(self, test):
        msg = html.escape('\n'.join(test.lines), True)
        attrs = {'message': msg, 'type': 'error'}
        return ET.Element('failure', attrs)


def parse_args():
    parser = argparse.ArgumentParser(description='Convert TSTP logs to XUnit')
    parser.add_argument('tarball', type=str, help='Path to test logs tarball')
    parser.add_argument('outdir', type=str, help='Root output directory path')
    return parser.parse_args()


def main():
    args = parse_args()
    print('extracting %s' % args.tarball)
    tf = tarfile.open(args.tarball, 'r')

    s = State()
    p = Parser(s)
    for m in tf.getmembers():
        f = tf.extractfile(m)
        if f and m.name.endswith('.log'):
            p.parse_buffer(m.name, f)

    CalculateTestTimes(s).run()

    outdir = args.outdir
    if not os.path.isdir(outdir):
        os.mkdir(outdir)

    e = XUnitEmitter(s, outdir)
    e.emit()


if __name__ == "__main__":
    main()

# vim: sts=4 sw=4 et cindent
