# Copyright (C) 2012 Timesys Corporation
#
# build.mk
#
# This file contains a bunch of macros to help build different
# types of tests.

################################################
# Simple C

# SIMPLE_C_BUILD generates rules for building a simple C test program.
#
# Additional CFLAGS can be given in a per-target rule; for example,
# an `example` test would use `example_CFLAGS`
#
# Macro arguments:
# $1 - name of test group
# $2 - name of test program

define SIMPLE_C_BUILD
$$($1_BIN_DIR)/$2: $$($1_OBJ_DIR)/$2
	mkdir -p $$(@D)
	install -m 0755 $$< $$@

$$($1_OBJ_DIR)/$2: $(TARGET_OBJ_DIR)/common/tstp.o $$($1_OBJ_DIR)/$2.o
	mkdir -p $$(@D)
	$$(CC) $$? -I$$(BASE_DIR)/include $$($2_CFLAGS) $$(CFLAGS) $$(LDFLAGS) -o $$@

$$($1_OBJ_DIR)/$2.o: $$($1_SRC_DIR)/$2.c
	mkdir -p $$(@D)
	cd $$(<D) && \
		$$(CC) -c $$< -I$$(BASE_DIR)/include $$(CFLAGS) $$($2_CFLAGS) -o $$@

endef


################################################
# Shell scripts

# SCRIPT_BUILD generates rules to build and install a test script that
# uses the `tstp.sh` shared script library.
#
# Macro arguments:
# $1 - name of test group
# $2 - name of test script

define SCRIPT_BUILD
$$($1_OBJ_DIR)/$2: $$($1_SRC_DIR)/$2
	mkdir -p $$(@D)
	sed -e 's,@TSTPLIB@,$$$${PREFIX:-$$(PREFIX)}/bin/tstp.sh,' $$< > $$@

$$($1_BIN_DIR)/$2: $$($1_OBJ_DIR)/$2
	mkdir -p $$(@D)
	install -m 0755 $$< $$@

endef


################################################
# Data files

# DATA_BUILD generates rules to install files that simply need to be
# copied into the filesystem.
#
# Macro arguments:
# $1 - name of test group
# $2 - name of test data file

define DATA_BUILD
$$($1_BIN_DIR)/$2: $$($1_SRC_DIR)/$2
	mkdir -p $$(@D)
	install -m 0644 $$< $$@

endef


################################################
# Test Group Makefiles

# INCLUDE_EXTRA_FRAGMENT loads an (optional) out-of-tree makefile fragment,
# allowing a build system to add new tests to existing test groups/packages.

define INCLUDE_EXTRA_FRAGMENT
-include $(BASE_DIR)/target/$1/$1.emk
endef

# SETUP_TARGET_DIRS defines variables that contain the source, build,
# and install directories required for building and install the test
# components for the given test group.
#
# Macro arguments:
# $1 - name of test group

define SETUP_TARGET_DIRS
$1_REL_DIR := $(1)
$1_SRC_DIR := $(BASE_DIR)/target/$$($1_REL_DIR)
$1_OBJ_DIR := $(TARGET_OBJ_DIR)/$$($1_REL_DIR)
$1_BIN_DIR := $(TARGET_BIN_DIR)/$$($1_REL_DIR)
endef

# SETUP_TESTS lists all of the test components required for the given
# test group, to create the list of build dependencies for the .
#
# Individual components may be excluded from the test group rule by
# including them in the `TSTP_DISABLED_TESTS` variable.
#
# Macro arguments:
# $1 - target class: SIMPLE_C, SCRIPT, DATA
# $2 - name of test group

define SETUP_TESTS
$(foreach test, $(filter-out $(TSTP_DISABLED_TESTS),$($2_$1)),$($2_BIN_DIR)/$(test))
endef

# SETUP_RULES defines all of the rules required for building and
# installing the various classes of test components for a test group
#
# Macro arguments:
# $1 - target class: SIMPLE_C, SCRIPT, DATA
# $2 - name of test group

define SETUP_RULES
$$(foreach app,$$($2_$1),$$(eval $$(call $1_BUILD,$2,$$(app))))
endef

# SETUP_STANDARD_RULES generates all of the targets and rules required
# for a test group Makefile framgment, adding the resulting group target
# to the main list of test targets.  The macro will look for variables
# containing the test components that need to be built/installed.  For
# example, an `example` test group could use the following `example.mk`:
# ```
# example_SIMPLE_C = example.c
# example_SCRIPTS = example.sh
# example_DATA = example.dat
# $(eval $(call SETUP_STANDARD_RULES example))
# ```
#
# Macro arguments:
# $1 - name of test group

define SETUP_STANDARD_RULES
$(eval $(call INCLUDE_EXTRA_FRAGMENT,$1))
$(eval $(call SETUP_TARGET_DIRS,$1))

$1: $(foreach grp,SIMPLE_C SCRIPT DATA,$(call SETUP_TESTS,$(grp),$1))

$(eval $(call SETUP_RULES,SIMPLE_C,$1))
$(eval $(call SETUP_RULES,SCRIPT,$1))
$(eval $(call SETUP_RULES,DATA,$1))

TSTP_TARGET_TESTS += $1
endef


# vim:set noexpandtab:
