#!/bin/sh -e

source "${TSTPLIB:-@TSTPLIB@}"

TESTS=""
TEST_SUITES=""

add_test() { TESTS="$TESTS${TESTS:+ }'$1'"; }
add_suites() { TEST_SUITES="$TEST_SUITES${TEST_SUITES:+ }$*"; }

opt_usage_i() { echo "Interactive tests only"; }
opt_parse_i() { add_suites interactive; }

opt_usage_a() { echo "Automated tests only"; }
opt_parse_a() { add_suites automated; }

opt_usage_s() { echo "Run the named suite(s)"; }
opt_parse_s() { add_suites "$1"; }

opt_usage_t() { echo "Only run the named test(s)"; }
opt_parse_t() { list_for_each add_test $1; }

opt_usage_L() { echo "Path to save log files"; }
opt_parse_L() { LOGDIR=$1; }

parse_opts "ais:t:L:" "$@"

[ "$TEST_SUITES" ] || TEST_SUITES="interactive automated"

DATE=$(date "+%Y%m%d%H%M%S")

LOGDIR=${LOGDIR:-$PREFIX/logs}/$DATE/

Normal='\e[0m'
BGreen='\e[1;32m'
BRed='\e[1;31m'

skip_test() {
	local name="'$1'"
	[ "$TESTS" -a "${TESTS/$name/}" = "$TESTS" ]
}

log_header() {
	echo "$1"
	echo ""
	header "$(date)"
}

_run_test() {
	if run "$@" >>$LOGFILE 2>&1; then
		RESULT=0
	else
		RESULT=1
	fi
}

start_test() {
	local TEST_NAME_SAFE

	TEST_NAME_SAFE=$(echo $TEST_NAME | sed -e 's/ /_/g')
	LOGFILE=$LOGDIR/$TEST_NAME_SAFE.log

	if skip_test "${TEST_NAME_SAFE}"; then
		debug "${TEST_NAME_SAFE}: skipping test"
		return 1
	fi

	log_header "$TEST_NAME" >$LOGFILE

	return 0
}

run_automated() {
	local TEST_NAME=$1
	local LOGFILE RESULT
	shift

	start_test || return 0
	_run_test "$@"
	log_result
}

_run_interactive() {
	local response

	echo
	header "Interactive Test: $TEST_NAME"

	while true; do
		echo "   $TEST_PROMPT_PRE" |tee -a $LOGFILE

		if [ -z "$TEST_PROMPT_POST" ]; then
			echo -n "   Press any key to continue"
			read
		else
			sleep 3
		fi

		_run_test "$@"

		[ "$TEST_PROMPT_POST" ] || { echo; break; }

		echo -n "   $TEST_PROMPT_POST (y/n/r[etry]): " |tee -a $LOGFILE
		read response
		echo
		echo "$response" >> $LOGFILE

		case "$response" in
		r|R|'')
			continue
			;;
		y|Y)
			RESULT=0
			;;
		n|N)
			RESULT=1
			;;
		*)
			echo "$response: unknown response, retrying..." >&2
			continue
		esac
		break
	done
}

run_interactive() {
	local TEST_NAME=$1
	local TEST_PROMPT_PRE=$2
	local TEST_PROMPT_POST=$3
	local LOGFILE RESULT
	shift 3

	start_test || return 0
	_run_interactive "$@"
	log_result
}

run_test() {
	local TEST_NAME=$1
	local TEST_PATH=$2
	local TEST_PARAMS=$3
	local TEST_PROMPT_PRE=$4
	local TEST_PROMPT_POST=$5
	local LOGFILE RESULT

	start_test || return 0

	if [ -n "$TEST_PROMPT_PRE" ]; then
		_run_interactive "$TEST_PATH" $TEST_PARAMS
	else
		_run_test "$TEST_PATH" $TEST_PARAMS
	fi

	log_result
}

log_result() {
	local color
	local logline

	if [ "$RESULT" = 0 ]; then
		RESULT="SUCCESS"
		color=$BGreen
	else
		RESULT="FAILURE"
		color=$BRed
	fi

	header "                             " >> $LOGFILE
	logline=$(printf "%-20s: $RESULT" "$TEST_NAME")
	echo "$logline"	>> $LOGFILE
	echo "$logline"	>> $SUITE_LOGFILE

	logline=$(printf "%-20s: ${color}$RESULT${Normal}" "$TEST_NAME")
	echo -e "$logline"
	SUMMARY="$SUMMARY$logline\n"
}

mkdir -p $LOGDIR

header "Test Run"
echo "Log directory: $LOGDIR"
echo ""

run_suite() {
	local name=$1
	local SUITE_LOGFILE="$LOGDIR/$name.log"
	local SUMMARY

	(
	header "Starting '$name' Tests"
	echo "Started: $(date)"
	echo
	) | tee $SUITE_LOGFILE

	source "$CONFDIR/$name.conf"

	if [ "$SUMMARY" ]; then
		echo
		header "'$name' Test Results"
		echo -e "$SUMMARY"
	fi

	(
	echo
	echo "Finished: $(date)"
	echo
	) | tee -a $SUITE_LOGFILE
}

run_suites() {
	local suite
	debug "Running test suites: $*"
	for suite in "$@"; do
		run_suite "$suite"
	done
}

run_suites $TEST_SUITES
