#!/bin/sh
# tstp.sh: common helper functions used by TSTP scripts

# name of current test
TEST=$(basename "$0")

# root installation path
PREFIX="${PREFIX:-@PREFIX@}"
# allow harness to run without being installed
PREFIX="${PREFIX:-.}"

# configuration file path
CONFDIR="$PREFIX/conf"

DEBUG=${DEBUG:-false}
# if DRY_RUN is true, prints the commands that would be run
DRY_RUN=${DRY_RUN:-false}

is_function() { type "$1" >/dev/null 2>&1; }

header() {
	echo "$*"
	echo "$*" | sed -e 's,.,=,g'
	echo
}

error() {
	echo "$TEST: ERROR: $@" >&2
	exit 1
}

warn() { echo "$TEST: WARNING: $*" >&2; }
info() { echo "$TEST: INFO: $*" >&2; }

debug() {
	! $DEBUG || echo "$TEST: $*" >&2
}


run() {
	echo "Running test: $*" >&2
	$DRY_RUN || "$@"
}

#####
# List helpers

list_prepend() {
	local var=$1
	shift
	eval $var="$*\${$var:+ }\$$var"
}

list_append() {
	local var=$1
	shift
	eval $var="\$$var\${$var:+ }$*"
}

list_for_each() {
	local func=$1
	local item
	shift
	for item in "$@"; do
		$func "$item"
	done
}

in_list() {
	local var=$1
	local item=$2
	eval [ "\${$var/\$item/}" != "\$$var" ]
}

#####
# Script cleanup helpers

CLEANUP_CALLBACKS=""

cleanup_add() {
	# By prepending these callbacks, they will be called in the
	# reverse order of being added, allowing nested (FILO) do/undo
	# action pairing.
	list_prepend CLEANUP_CALLBACKS "$@"
}

cleanup_handler() {
	local f
	for f in $CLEANUP_CALLBACKS; do
		$f
	done
}

trap cleanup_handler EXIT

#####
# Script libraries

LIBPATH="$PREFIX/bin/lib $PREFIX/bin"

find_library() {
	local name=$1
	local path file
	for path in $LIBPATH; do
		file="$path/$name.sh"
		if [ -f "$file" ]; then
			echo "$file"
			return
		fi
	done
	error "$name.sh: script library not found"
}

load_libraries() {
	local lib libfile libinit
	for lib in "$@"; do
		debug "$lib.sh: loading library..."
		libfile=$(find_library "$lib")
		source "$libfile"
		libinit="${lib}_lib_init"
		if is_function "$libinit"; then
			debug "$lib.sh: initializing library..."
			$libinit
		fi
	done
}

load_libraries opts deps
