#!/bin/sh
# test_gpio_sysfs.sh: uses sysfs to export/configure/read GPIO pins
#
# This script reads lines from a file in the following format:
# ```
#    Pin|Name|Direction|Interrupt
# ```
# where `pin` is the GPIO number, `name` is the symbolic name (e.g.
# the name of the line on the schematic), `direction` is either `in`
# or `out`, and `interrupt` specifies an (optional) interrupt type.
#
# The script must be given a command:
#   list: lists the pins configured for use by sysfs
#   export: makes the GPIO pins available via sysfs
#   setup: configures the pins in sysfs for testing
#   read: reads the pins
#   unexport: frees the GPIO pins from sysfs control
#   all: performs all of the above commands in sequence
#
# Hardware: GPIO pins to test
# Packages: none

source "@TSTPLIB@"

GPIOS=${GPIOS:-/usr/tstp/bin/gpio/gpio_sysfs.pins}

GPIODIR=/sys/class/gpio

#####
# Pin commands

do_list() {
	echo "gpio$pin: $name is an ${dir}put"
	[ -z "$irq" ] || echo "gpio$pin: enabling $irq edge detection"
}

do_export() {
	[ -d "$GPIODIR/gpio$pin" ] || echo $pin >$GPIODIR/export
	debug "gpio$pin: exported"
	do_setup
}
do_unexport() {
	[ ! -d "$GPIODIR/gpio$pin" ] || echo $pin >$GPIODIR/unexport
	debug "gpio$pin: unexported"
}

do_setup() {
	echo $dir >$GPIODIR/gpio$pin/direction
	if [ "$irq" ]; then
		debug "gpio$pin: $irq edge interrupts enabled"
		echo $irq >$GPIODIR/gpio$pin/edge
	fi
}

do_read() {
	local val
	val=$(cat $GPIODIR/gpio$pin/value)
	echo "gpio$pin: $name ${dir}put is $val"
}

#####
# File parsing

read_line() { IFS= read -r $1; }
parse_line() { IFS='|' read -r $@; }

do_pin() {
	local pin name dir irq
	parse_line pin name dir irq

	[ "$pin" ] || return 0
	if [ -z "$name" ]; then
		echo "gpio $pin: name not specified" >&2
		return
	fi
	if [ -z "$dir" ]; then
		echo "$name: gpio $pin: direction not specified" >&2
		return
	fi

	$do_one "$@"
}

do_file_read() {
	local line
	while read_line line; do
		if [ "${line#\#}" = "$line" ]; then
			echo "$line" | do_pin "$@"
		fi
	done
}

do_file() {
	do_file_read "$@" <"$GPIOS"
}

#####
# Command handling

do_cmd() {
	local cmd=$1
	[ "$cmd" ] || usage "missing command"
	shift

	[ -f "$GPIOS"  ] || error "$GPIOS: pin configuration not found"

	local do_one="do_$cmd"

	case "$cmd" in
	export|unexport|setup|list|read)
		do_file
		;;
	all)
		do_cmd export
		do_cmd setup
		do_cmd read
		do_cmd unexport
		;;
	*)
		usage "unknown command: $cmd"
		;;
	esac
}

#####
# Option parsing

opt_usage_G() { echo "Use a different pin configuration file"; }
opt_parse_G() { GPIOS=$1; }

args_usage() { echo "<command> [...]"; }
args_help() {
	cat <<EOF

Commands:
    export - Exports pins via sysfs
  unexport - Unexports pins from sysfs
     setup - Configures pins as inputs/outputs
      list - Lists all pins and configuration
      read - Reads current values of all pins
       all - Runs export, setup, read, and unexport commands

Current pin config file: $GPIOS
EOF
}
args_parse() {
	do_cmd "$@"
}

parse_opts "G:" "$@"

[ "$*" ] || usage "missing command"
