#!/bin/sh -e
# test_i2c.sh: tests i2c buses and devices in various ways
#
# The 'detect' command runs `i2cdetect` to list all of the I2C buses,
# then it uses that utility to scan each bus for devices.  For each bus,
# it outputs a table with the list of detected devices on that bus.
#
# The 'scan' command runs `i2cdetect` on the selected I2C bus,
# Outputs a table with the list of detected devices on that bus.
#
# The 'dump' command runs `i2cdump` to dump all of the registers for
# a single I2C device.  This can be useful for device not detected by
# the `i2cdetect` command.
#
# The 'read' command runs `i2cget` to read a single register from an I2C
# device, optionally comparing it with an expected value.
#
# The 'write' command runs `i2cset` to write a data register on an I2C
# device.  It then reads the bus for a response, optionally comparing it
# with an expected value.
#
# Dependencies: i2cdetect i2cdump i2cget i2cset

source @TSTPLIB@

args_usage() { echo "<cmd> ..."; }
args_help() {
	cat <<EOF
Commands:
  detect			Detect devices on all i2c buses
  scan <bus>			Detect devices on selected i2c bus
  dump <bus> <addr>		Dump all addresses for the given bus device
  read <bus> <addr> <reg> [<value>]
				Read a device register; if a value is given,
				verifies that the result matches it.
  write <bus> <addr> <reg> [<value>]
				Write a device register; if a value is given,
				verifies that the result matches it.
EOF
}

do_i2c_detect() {
	local buses n_buses bus test_error
	local test_error=0
	#Get registered i2c buses indexes
	buses=$(i2cdetect -l | cut -f1 | cut -d'-' -f2)
	echo "$buses"
	for bus in $buses; do
		run i2cdetect -y $bus || error "i2c-$bus: detection failed"
	done
}

do_i2c_scan() {
	local bus=$1
	run i2cdetect -y $bus || ( echo "i2c-$bus: scan failed"; exit 1)
}

check_i2c_device() {
	local bus=$1
	dev="/dev/i2c-$bus"
	[ -e "$dev" ] || error "$dev: not found"
}


do_i2c_dump() {
	local bus=$1
	local addr=$2
	[ "$bus" -a "$addr" ] || usage
	check_i2c_device "$bus"
	run i2cdump -y "$bus" "$addr"
}

do_i2c_read() {
	local bus=$1
	local addr=$2
	local reg=$3
	local value=$4
	local response

	check_i2c_device "$bus"
	[ "$bus" -a "$addr" -a "$reg" ] || usage "missing arguments"
	_do_i2c_read $bus $addr $reg
}

_do_i2c_read() {
	local bus=$1
	local addr=$2
	local reg=$3

	response=$(run i2cget -y -f $bus $addr $reg || echo "FAIL")
	if [ "$value" ]; then
		[[ "$response" == "$value" ]]
	else
		[ "$response" != "FAIL" ]
	fi
}

do_i2c_write() {
	local bus=$1
	local addr=$2
	local reg=$3
	local value=$4
	local response

	check_i2c_device "$bus"
	[ "$bus" -a "$addr" -a "$reg" ] || usage "missing arguments"

	run i2cset -y -f $bus $addr $reg
	_do_i2c_read $bus $addr
}

do_cmd() {
	cmd=$1
	[ "$cmd" ] || usage "missing command"
	shift

	case "$cmd" in
	detect)
		dep_check_programs i2cdetect
		do_i2c_detect
		;;
	scan)
		dep_check_programs i2cdetect
		do_i2c_scan $1
		;;
	dump)
		dep_check_programs i2cdump
		do_i2c_dump "$@"
		;;
	read)
		dep_check_programs i2cget
		do_i2c_read "$@"
		;;
	write)
		dep_check_programs i2cget i2cset
		do_i2c_write "$@"
		;;
	-h|help)
		usage
		;;
	*)
		usage "unknown command"
	esac

	exit $?
}

args_parse() {
	do_cmd "$@"
}

parse_opts "" "$@"
