#!/bin/sh
# test_card.sh: tests an SD or MMC card
# This script mounts the card and copies files to check the md5sum.
# If the card is not properly partitioned, it will format the card.

set -e

source "@TSTPLIB@"

load_libraries files

CARD_TYPE=auto
CARD_NODE=
ROOTFS_SIZE=400M
MOUNT_POINT=/mnt/card
TEST_PART=1

opt_parse_t() { CARD_TYPE=$1; }
opt_usage_t() { echo "type of card (SD or MMC)"; }

opt_parse_s() { ROOTFS_SIZE=$1; }
opt_usage_s() { echo "size of root filesystem (default: $ROOTFS_SIZE)"; }

opt_parse_p() { TEST_PART=$1; }
opt_usage_p() { echo "partition for r/w test (default p1)"; } 

args_usage() { echo "<node>"; }
args_help() {
	cat <<EOF
  <node>	Card device node
EOF
}
args_parse() {
	CARD_NODE=$1
}

parse_opts "t:s:p:" "$@"

[ "$CARD_NODE" ] || error "card device node must be provided"

NODE=/dev/mmcblk${CARD_NODE}
[ -b "$NODE" ] || error "$NODE is not a block device"

card_is_sd() { dmesg | grep 'SD card' >/dev/null; }
card_is_mmc() { dmesg | grep 'MMC card' >/dev/null; }

card_type_detect() {
	if card_is_sd; then
		CARD_TYPE=SD
	elif card_is_mmc; then
		CARD_TYPE=MMC
	else
		error "unable to detect card type"
	fi
}

[ "$CARD_TYPE" != auto ] || card_type_detect
case "$CARD_TYPE" in
SD)
	;;
MMC)
	;;
*)
	error "$CARD_TYPE: unknown card type"
	;;
esac

card_partition() {
	info "$CARD_TYPE: partition missing, creating..."
	sed -e 's/\s*\([\+0-9a-zA-Z]*\).*/\1/' << EOF | run fdisk $NODE
     o # clear the in memory partition table
     n # new partition
     p # primary partition
     1 # partition number 2
     2048  # default - start at beginning of disk 
     +${ROOTFS_SIZE}  # root partition size
     w # write the partition table
     q # and we're done
EOF
	run mkfs.ext4 -F ${NODE}p1 -L root
	info "$CARD_TYPE: partition created."
}

echo "--------$CARD_TYPE Testing-----------"

info "checking card partitions"
if fdisk -l ${NODE} | grep ${NODE}p${TEST_PART} >/dev/null 2>&1; then
	info "$CARD_TYPE: partition exists"
else
	# back to default
	TEST_PART=1
	card_partition
fi

card_mount() {
	run mkdir -p $MOUNT_POINT
	run mount ${NODE}p${TEST_PART} $MOUNT_POINT
	cleanup_add card_umount
}
card_umount() {
	rm -f $BFILE
	run umount ${NODE}p${TEST_PART}
}
card_is_mounted() {
	df -h | grep ${NODE}p${TEST_PART}
}

# Already mounted ?
if card_is_mounted; then
	card_umount
fi
card_mount

mktempfile AFILE
mktempfile BFILE -p $MOUNT_POINT

run dd if=/dev/urandom of=$AFILE bs=1M count=1

info "Perform r/w operation on $CARD_TYPE"
a_hash=$(run md5sum $AFILE | cut -d' ' -f1)
run cp $AFILE $BFILE
b_hash=$(run md5sum $BFILE | cut -d' ' -f1)

if [ "$a_hash" = "$b_hash" ]; then
	info "$CARD_TYPE: hash matched ($a_hash)"
else
	error "$CARD_TYPE: hash mismatch ($a_hash != $b_hash)"
fi
