#!/bin/sh
# flash.sh: common helper functions used by MTD flash tests

dep_check_alternatives FLASH_ERASE flash_erase flash_eraseall

mtd_erase() {
	case "$FLASH_ERASE" in
	*/flash_erase)
		$FLASH_ERASE $DEVICE 0 0
		;;
	*/flash_eraseall)
		$FLASH_ERASE -q $DEVICE
		;;
	esac
}

# mtd_parse_opts() handles the command line arguments for an MTD test
# It sets the follow global variables:
# - `PART`: The MTD partition number specified on the command line
# - `DEVICE`: Full path of the MTD character device file
# - `SYSFSDIR`: Full path to the sysfs directory for the given MTD node
# - `PARTSIZE`: Full size of the MTD partition
# - `ERASESIZE`: Erase block size for the MTD partition
# - `ERASEBLOCKS`: Number of blocks to erase the entire MTD partition
mtd_parse_opts() {
	PART=$1
	[ "$PART" ] || error "partition argument missing"

	DEVICE=/dev/mtd$PART
	[ -c "$DEVICE" ] || error "$DEVICE: not found"

	SYSFSDIR=/sys/class/mtd/mtd$PART/
	[ -d "$SYSFSDIR" ] || error "$SYSFSDIR: not found"

	PARTSIZE=$(cat $SYSFSDIR/size)
	ERASESIZE=$(cat $SYSFSDIR/erasesize)
	ERASEBLOCKS=$((PARTSIZE / ERASESIZE))
}

# mtd_test_start() checks for required programs and displays partition info
mtd_test_start() {
	local type=$1

	dep_check_programs dd
	case "$type" in
	NOR)
		dep_check_programs flashcp
		;;
	NAND)
		dep_check_programs nandwrite nanddump
		;;
	*)
		error "$type: unknown test type"
		;;
	esac

	# print summary of MTD under test
	echo "MTD $DEVICE:"
	echo "  $type Flash"
	printf "  Total Size: 0x%x (%d)\n" $PARTSIZE $PARTSIZE
	printf "  Erase Size: 0x%x (%d)\n" $ERASESIZE $ERASESIZE
	printf "  Number Blocks: %d\n" $ERASEBLOCKS
	echo
}

# mtd_test_input_ones() creates a file containing all ones
mtd_test_input_ones() {
	dd if=/dev/zero bs=$1 count=$2 | tr "\000" "\377"
}

# mtd_test_input_random() creates a test file containing random data
mtd_test_input_random() {
	dd if=/dev/urandom bs=$1 count=$2
}

# mtd_test_run() performs a single MTD test
# This function invokes three commands indirectly through three variables:
# - `MTD_TEST_INPUT`: Outputs test file contents to stdout
# - `MTD_TEST_WRITE`: Writes $INFILE to $DEVICE
# - `MTD_TEST_READ`: Reads $DEVICE into $OUTFILE
# After running these in sequence, $INFILE and $OUTFILE are compared.
mtd_test_run() {
	local name=$1
	local INFILE OUTFILE
	mktempfiles INFILE OUTFILE

	header "$name Test"

	$MTD_TEST_INPUT >$INFILE

	echo "+++ Erasing flash"
	mtd_erase $DEVICE

	echo "+++ Writing file to flash"
	$MTD_TEST_WRITE

	echo "+++ Reading flash"
	$MTD_TEST_READ

	if files_equal "$INFILE" "$OUTFILE"; then
		echo "+++ $name Test Passed"
	else
		error "+++ $name Test Failed"
	fi
}
