#!/bin/sh -e
# test_pm-cpufreq.sh: exercises the cpufreq driver using sysfs
#
# This test exercises the CPU frequency scaling support, using each of
# the available governors.
#
# If the `userspace` governor is available, the CPU will be exercised at
# each operating point.
#
# Kernel: cpufreq support, userspace governor (for manual speed tests)
# Hardware: none
# Packages: dd md5sum

source "@TSTPLIB@"

dep_check_programs dd md5sum

CPU=$1

if [ -z "$CPU" ]; then
    CPU=0
fi

CPUFREQDIR=/sys/bus/cpu/devices/cpu$CPU/cpufreq/

if ! [ -d "$CPUFREQDIR" ]; then
    echo "CPU Frequency Scaling is not available for this processor!"
    exit 1
fi

MINFREQ=$(cat ${CPUFREQDIR}/cpuinfo_min_freq)
MAXFREQ=$(cat ${CPUFREQDIR}/cpuinfo_max_freq)
GOVERNERS=$(cat ${CPUFREQDIR}/scaling_available_governors)
FREQUENCIES=$(cat ${CPUFREQDIR}/scaling_available_frequencies)

CURGOVFILE=${CPUFREQDIR}/scaling_governor
CURFREQFILE=${CPUFREQDIR}/scaling_cur_freq
SETFREQFILE=${CPUFREQDIR}/scaling_setspeed

TESTFILE=/tmp/testfile.dat
TESTFILESIZE=16

trap cleanup EXIT

cleanup() {
	rm -f "$TESTFILE"
}

dd if=/dev/urandom of=$TESTFILE bs=1M count=$TESTFILESIZE > /dev/null 2>&1

echo "Testing CPU Frequency on CPU$CPU"
echo "  Available Governers: $GOVERNERS"
echo "  Available Frequencies: $FREQUENCIES"
echo ""

get_governor() {
	cat $CURGOVFILE
}

set_governor() {
	echo $1 > $CURGOVFILE
}

get_frequency() {
	cat $CURFREQFILE
}

set_frequency() {
	echo $1 > $SETFREQFILE
}

test_frequency() {
	local freq=$1
	if ! set_frequency $freq || [ "$(get_frequency)" != "$freq" ]; then
		echo "error: unable to set frequency to $freq"
		return
	fi
	echo "Running at $freq Hz"
	time md5sum $TESTFILE >/dev/null
	echo ""
}

# Test manual frequency setting (if supported)
test_frequencies() {
	local freq
	if ! set_governor userspace; then
		echo "info: userspace governor is missing," \
		       "skipping manual speed tests"
		return
	fi
	echo "Manual Set Speed Tests"
	echo "======================"
	for freq in $FREQUENCIES; do
		test_frequency $freq
	done

	echo ""
}

test_frequencies

test_governor() {
	local gov=$1
	local elapsed

	set_governor $gov
	if [ "$(get_governor)" != "$gov" ]; then
		echo "error: unable to set governer to $gov"
		return
	fi
	echo "Governer: $gov"

	# Give system some time to adjust to new governer
	sleep 2

	echo "    Unloaded speed: $(get_frequency)"
	elapsed=$(time md5sum $TESTFILE  2>&1 >/dev/null)

	echo "    Loaded speed: $(get_frequency)"
	echo "$elapsed"

	echo ""
}

test_governors() {
	local gov

	echo "Governer Tests"
	echo "=============="

	for gov in $GOVERNERS; do
		test_governor $gov
	done
}

test_governors
