#!/bin/sh -e
# rtc.sh:  test the RTC by manipulating the system time and hardware clock
#
# This script test the RTC device by changing the system time and
# hardware clock, ensuring that the hardware clock can be written and
# read correctly.
#
# Kernel: rtc support
# Dependencies: cut date hwclock

source "@TSTPLIB@"

dep_check_programs date hwclock

is_busybox_symlink() { readlink -f $1 | grep -q busybox; }
set_busybox_flag() {
	local file=$1
	local var=$2
	local result
	is_busybox_symlink $file && result=true || result=false
	eval "$var=$result"
}

set_busybox_flag /sbin/hwclock hwclock_is_busybox
set_busybox_flag /bin/date date_is_busybox

rtcdev=/dev/$1

if [ $# -ne 1 ]; then
   echo "rtc.sh <rtc dev port>"
   echo "e.g.:"
   echo "   rtc.sh rtc0"
   exit 1
fi

# compare two times for equality; they must be formatted identically
compare_times() {
	local sys=$1
	local hw=$2
	# on some systems, there may be a difference of a couple of seconds
	# but the test should be considered good anyway
	if [ "$sys" != "$hw" ]; then
		if [ "$sys" -gt $(expr "$hw" + 2) ] || [ "$sys" -lt $(expr "$hw" - 2) ]; then
			echo "FAIL"
			error "times differ: $sys != $hw"
		fi
	fi
}

# save the original time in settable format
t_saved=$(date +%m%d%H%M%Y.%S)

restore_time() {
	echo -n "Restoring system time to "
	date $t_saved
	echo "Restoring hardware clock to system time"
	hwclock --systohc --rtc=$rtcdev
}

cleanup_add restore_time

# report the system time in comparable format
systime() {
	date +%s
}

set_systime() {
	date $1 >/dev/null
}

# report the hwclock time in comparable format
hwtime() {
	if $date_is_busybox; then
		# TODO: busybox may not be configured to support -D FMT
		date -d "$(hwclock -u)" -D "%a %b %d %H:%M:%S %Y" +%s
	else
		date --date="$(hwclock -u)" +%s
	fi
}

echo "Current System Time is: $(systime)"
echo "Current Hardware Clock is: $(hwtime)"

echo -n "Changing system time to Nov 23 23:59:00 2037... "
set_systime 112323592037
sys1=$(systime)
echo "PASS"

echo -n "Syncing system time to hardware clock... "
hwclock --systohc --rtc=$rtcdev
echo "PASS"

echo -n "Checking updated hardware clock... "
hw1=$(hwtime)
compare_times "$sys1" "$hw1"
echo "PASS"

echo -n "Changing system time to Jan 01 01:01:00 2001... "
set_systime 010101012001
echo "PASS"

echo -n "Syncing hardware clock to system time... "
hwclock --hctosys --rtc=$rtcdev
sys2=$(systime)
hw2=$(hwtime)
echo "PASS"

echo -n "Verifying system time... "
compare_times "$sys2" "$hw2"
echo "PASS"
